/*  energy subroutines, to be used in sff.c    */

REAL_T econs( REAL_T *x, REAL_T *f )
{
	int i;
	REAL_T e_cons, rx, ry, rz;

	e_cons = 0.0;
	for (i = 0; i < prm->Natom; i++) {
		if (constrained[i]) {
			rx = x[3 * i] - x0[3 * i];
			ry = x[3 * i + 1] - x0[3 * i + 1];
			rz = x[3 * i + 2] - x0[3 * i + 2];
			e_cons += wcons * (rx * rx + ry * ry + rz * rz);
			f[3 * i] += 2. * wcons * rx;
			f[3 * i + 1] += 2. * wcons * ry;
			f[3 * i + 2] += 2. * wcons * rz;
		}
	}
	return (e_cons);
}

REAL_T ebond( int nbond, int *a1, int *a2, int *atype,
	REAL_T *Rk, REAL_T *Req, REAL_T *x, REAL_T *f )
{
	int i, at1, at2, atyp;
	REAL_T e_bond, r, rx, ry, rz, r2, s, db, df, e;

	e_bond = 0.0;
	for (i = 0; i < nbond; i++) {
		at1 = a1[i];
		at2 = a2[i];
		atyp = atype[i] - 1;
		rx = x[at1] - x[at2];
		ry = x[at1 + 1] - x[at2 + 1];
		rz = x[at1 + 2] - x[at2 + 2];
		r2 = rx * rx + ry * ry + rz * rz;
		s = sqrt(r2);
		r = 2.0 / s;
		db = s - Req[atyp];
		df = Rk[atyp] * db;
		e = df * db;
		e_bond += e;
		df *= r;
		f[at1 + 0] += rx * df;
		f[at1 + 1] += ry * df;
		f[at1 + 2] += rz * df;
		f[at2 + 0] -= rx * df;
		f[at2 + 1] -= ry * df;
		f[at2 + 2] -= rz * df;
	}
	if (e_debug)
		EXPR("%9.3f", e_bond);
	return (e_bond);
}

REAL_T eangl( int nang, int *a1, int *a2, int *a3, int *atype,
	REAL_T *Tk, REAL_T *Teq, REAL_T *x, REAL_T *f )
{
	int i, atyp, at1, at2, at3;
	REAL_T dxi, dyi, dzi, dxj, dyj, dzj, ri2, rj2, ri, rj, rir, rjr;
	REAL_T dxir, dyir, dzir, dxjr, dyjr, dzjr, cst, at, da, df, e, e_theta;
	REAL_T xtmp, dxtmp, ytmp, dytmp, ztmp, dztmp;

	e_theta = 0.0;
	for (i = 0; i < nang; i++) {
		at1 = a1[i];
		at2 = a2[i];
		at3 = a3[i];
		atyp = atype[i] - 1;

		dxi = x[at1] - x[at2];
		dyi = x[at1 + 1] - x[at2 + 1];
		dzi = x[at1 + 2] - x[at2 + 2];
		dxj = x[at3] - x[at2];
		dyj = x[at3 + 1] - x[at2 + 1];
		dzj = x[at3 + 2] - x[at2 + 2];

		ri2 = dxi * dxi + dyi * dyi + dzi * dzi;
		rj2 = dxj * dxj + dyj * dyj + dzj * dzj;
		ri = sqrt(ri2);
		rj = sqrt(rj2);
		rir = 1. / ri;
		rjr = 1. / rj;

		dxir = dxi * rir;
		dyir = dyi * rir;
		dzir = dzi * rir;
		dxjr = dxj * rjr;
		dyjr = dyj * rjr;
		dzjr = dzj * rjr;

		cst = dxir * dxjr + dyir * dyjr + dzir * dzjr;
		if (cst > 1.0)
			cst = 1.0;
		if (cst < -1.0)
			cst = -1.0;

		at = acos(cst);
		da = at - Teq[atyp];
		df = da * Tk[atyp];
		e = df * da;
		e_theta = e_theta + e;
		df = df + df;
		at = sin(at);
		if (at > 0 && at < 1.e-3)
			at = 1.e-3;
		else if (at < 0 && at > -1.e-3)
			at = -1.e-3;
		df = -df / at;

		xtmp = df * rir * (dxjr - cst * dxir);
		dxtmp = df * rjr * (dxir - cst * dxjr);
		ytmp = df * rir * (dyjr - cst * dyir);
		dytmp = df * rjr * (dyir - cst * dyjr);
		ztmp = df * rir * (dzjr - cst * dzir);
		dztmp = df * rjr * (dzir - cst * dzjr);

		f[at1 + 0] += xtmp;
		f[at3 + 0] += dxtmp;
		f[at2 + 0] -= xtmp + dxtmp;
		f[at1 + 1] += ytmp;
		f[at3 + 1] += dytmp;
		f[at2 + 1] -= ytmp + dytmp;
		f[at1 + 2] += ztmp;
		f[at3 + 2] += dztmp;
		f[at2 + 2] -= ztmp + dztmp;

	}
	if (e_debug)
		EXPR("%9.3f", e_theta);
	return (e_theta);
}

REAL_T ephi( int nphi, int *a1, int *a2, int *a3, int *a4, int *atype,
	REAL_T *Pk, REAL_T *Pn, REAL_T *Phase, REAL_T *x, REAL_T *f )
{
	REAL_T e, co, den, co1, uu, vv, uv, ax, bx, cx, ay, by, cy, az, bz, cz;
	REAL_T a0x, b0x, c0x, a0y, b0y, c0y, a0z, b0z, c0z, a1x, b1x;
	REAL_T a1y, b1y, a1z, b1z, a2x, b2x, a2y, b2y, a2z, b2z;
	REAL_T dd1x, dd2x, dd3x, dd4x, dd1y, dd2y, dd3y, dd4y, dd1z, dd2z,
		dd3z, dd4z;
	REAL_T df, aa, bb, cc, ab, bc, ac, ktot;
	REAL_T ktors1, ktors2, ktors3, ktors4, phase, e_tors;
	int i, at1, at2, at3, at4, atyp;
	REAL_T ux, uy, uz, vx, vy, vz, delta, phi, dx1, dy1, dz1, yy, pi;
	pi = 3.1415927;

	e_tors = 0.0;
	for (i = 0; i < nphi; i++) {

		at1 = a1[i];
		at2 = a2[i];
		at3 = abs(a3[i]);
		at4 = abs(a4[i]);
		atyp = atype[i] - 1;

		ax = x[at2 + 0] - x[at1 + 0];
		ay = x[at2 + 1] - x[at1 + 1];
		az = x[at2 + 2] - x[at1 + 2];
		bx = x[at3 + 0] - x[at2 + 0];
		by = x[at3 + 1] - x[at2 + 1];
		bz = x[at3 + 2] - x[at2 + 2];
		cx = x[at4 + 0] - x[at3 + 0];
		cy = x[at4 + 1] - x[at3 + 1];
		cz = x[at4 + 2] - x[at3 + 2];

#		define DOT(a,b,c,d,e,f) a*d + b*e + c*f
		ab = DOT(ax, ay, az, bx, by, bz);
		bc = DOT(bx, by, bz, cx, cy, cz);
		ac = DOT(ax, ay, az, cx, cy, cz);
		aa = DOT(ax, ay, az, ax, ay, az);
		bb = DOT(bx, by, bz, bx, by, bz);
		cc = DOT(cx, cy, cz, cx, cy, cz);

		uu = (aa * bb) - (ab * ab);
		vv = (bb * cc) - (bc * bc);
		uv = (ab * bc) - (ac * bb);

		den = 1.0 / sqrt(fabs(uu * vv));
		co = uv * den;
		co1 = 0.5 * co * den;

		a0x = -bc * bx + bb * cx;
		a0y = -bc * by + bb * cy;
		a0z = -bc * bz + bb * cz;

		b0x = ab * cx + bc * ax - 2. * ac * bx;
		b0y = ab * cy + bc * ay - 2. * ac * by;
		b0z = ab * cz + bc * az - 2. * ac * bz;

		c0x = ab * bx - bb * ax;
		c0y = ab * by - bb * ay;
		c0z = ab * bz - bb * az;

		a1x = 2. * uu * (-cc * bx + bc * cx);
		a1y = 2. * uu * (-cc * by + bc * cy);
		a1z = 2. * uu * (-cc * bz + bc * cz);

		b1x = 2. * uu * (bb * cx - bc * bx);
		b1y = 2. * uu * (bb * cy - bc * by);
		b1z = 2. * uu * (bb * cz - bc * bz);

		a2x = -2. * vv * (bb * ax - ab * bx);
		a2y = -2. * vv * (bb * ay - ab * by);
		a2z = -2. * vv * (bb * az - ab * bz);

		b2x = 2. * vv * (aa * bx - ab * ax);
		b2y = 2. * vv * (aa * by - ab * ay);
		b2z = 2. * vv * (aa * bz - ab * az);

		dd1x = (a0x - a2x * co1) * den;
		dd1y = (a0y - a2y * co1) * den;
		dd1z = (a0z - a2z * co1) * den;

		dd2x = (-a0x - b0x - (a1x - a2x - b2x) * co1) * den;
		dd2y = (-a0y - b0y - (a1y - a2y - b2y) * co1) * den;
		dd2z = (-a0z - b0z - (a1z - a2z - b2z) * co1) * den;

		dd3x = (b0x - c0x - (-a1x - b1x + b2x) * co1) * den;
		dd3y = (b0y - c0y - (-a1y - b1y + b2y) * co1) * den;
		dd3z = (b0z - c0z - (-a1z - b1z + b2z) * co1) * den;

		dd4x = (c0x - b1x * co1) * den;
		dd4y = (c0y - b1y * co1) * den;
		dd4z = (c0z - b1z * co1) * den;

		if (prm->Nhparm && a3[i] < 0) {
/*   here we will use a quadratic form for the improper torsion  */

			co = co > 1.0 ? 1.0 : co;
			co = co < -1.0 ? -1.0 : co;

			phi = acos(co);

/*
	now calculate sin(phi) because cos(phi) is symmetric, so
	we can decide between +-phi.
*/

			ux = ay * bz - az * by;
			uy = az * bx - ax * bz;
			uz = ax * by - ay * bx;

			vx = by * cz - bz * cy;
			vy = bz * cx - bx * cz;
			vz = bx * cy - by * cx;

			dx1 = uy * vz - uz * vy;
			dy1 = uz * vx - ux * vz;
			dz1 = ux * vy - uy * vx;

			dx1 = DOT(dx1, dy1, dz1, bx, by, bz);
			if (dx1 < 0.0)
				phi = -phi;

			delta = phi - Phase[atyp];
			delta = delta > pi ? pi : delta;
			delta = delta < -pi ? -pi : delta;

			df = Pk[atyp] * delta;
			e = df * delta;
			e_tors += e;
			yy = sin(phi);

/*
	Decide what expansion to use
	Check first for the "normal" expression, since it will be
	the most used

	the 0.001 value could be lowered for increased precision.
	This insures ~1e-05% error for sin(phi)=0.001
*/

			if (fabs(yy) > 0.001) {
				df = -2.0 * df / yy;
			} else {
				if (fabs(delta) < 0.10) {
					if (Phase[atyp] == 0.0) {
						df = -2.0 * Pk[atyp] * (1 + phi * phi / 6.0);
					} else {
						if (fabs(Phase[atyp]) == pi) {
							df = 2.0 * Pk[atyp] * (1 +
												   delta * delta / 6.0);
						}
					}
				} else {
					if ((phi > 0.0 && phi < (pi / 2.0)) ||
						(phi < 0.0 && phi > -pi / 2.0))
						df = df * 1000.;
					else
						df = -df * 1000.;
				}
			}
		} else {
		  multi_term:
			ktors1 = 0.;
			ktors2 = 0.;
			ktors3 = 0.;
			ktors4 = 0.;
			switch ((int) fabs(Pn[atyp])) {
			case 1:
				ktors1 = Pk[atyp];
				break;
			case 2:
				ktors2 = Pk[atyp];
				break;
			case 3:
				ktors3 = Pk[atyp];
				break;
			case 4:
				ktors4 = Pk[atyp];
				break;
			default:
				fprintf(stderr, "bad value for Pn: %d %d %d %d %8.3f\n",
						at1, at2, at3, at4, Pn[atyp]);
				exit(1);
			}
			if (fabs(Phase[atyp] - 3.142) < 0.01)
				phase = -1.0;
			else
				phase = 1.0;

			e = 4. * ktors3 * co * co + 2. * ktors2 * co + ktors1 -
				3. * ktors3 + 8. * ktors4 * co * (co * co - 1.);
			e = e * co - ktors2 + ktors4;
			ktot = ktors1 + ktors2 + ktors3 + ktors4;
			e = ktot + phase * e;
			e_tors += e;

			df = phase * (12. * ktors3 * co * co + 4. * ktors2 * co +
						  ktors1 - 3. * ktors3 +
						  32. * ktors4 * co * co * co - 16. * ktors4 * co);

		}

		f[at1 + 0] += df * dd1x;
		f[at1 + 1] += df * dd1y;
		f[at1 + 2] += df * dd1z;

		f[at2 + 0] += df * dd2x;
		f[at2 + 1] += df * dd2y;
		f[at2 + 2] += df * dd2z;

		f[at3 + 0] += df * dd3x;
		f[at3 + 1] += df * dd3y;
		f[at3 + 2] += df * dd3z;

		f[at4 + 0] += df * dd4x;
		f[at4 + 1] += df * dd4y;
		f[at4 + 2] += df * dd4z;

#ifdef PRINT_EPHI
		printf("%4d %4d %4d %4d %4d %9.4f\n", i + 1, at1 / 3, at2 / 3,
			   at3 / 3, at4 / 3, e);
		printf("%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f\n",
			   -df * dd1x, -df * dd1y, -df * dd1z, -df * dd2x, -df * dd2y,
			   -df * dd2z, -df * dd3x, -df * dd3y);
		printf("%10.5f%10.5f%10.5f%10.5f\n", -df * dd3z, -df * dd4x,
			   -df * dd4y, -df * dd4z);
#endif
		if (Pn[atyp] < 0) {
			atyp++;
			goto multi_term;
		}

	}
	if (e_debug)
		EXPR("%9.3f", e_tors);
	return (e_tors);
}

REAL_T eimphi( int nphi, int *a1, int *a2, int *a3, int *a4, int *atype,
	REAL_T *Pk, REAL_T *Peq, REAL_T *x, REAL_T *f )
{

/*
	calculate improper dihedral energies and forces according
	to T.Schlick, J.Comput.Chem, vol 10, 7 (1989) 
	with local variant (no delta funcion involved)

   If |sin(phi)|.gt.1e-03 use the angle directly
       V = kimp ( phi -impeq ) ^2
	dV/dcos(phi) = -2*k/sin(phi)

   If |sin(phi)|.lt.1e-03 and (phi-Peq).lt.0.1
   and cnseq(phi)=0.
	V = kimp (phi - impeq)^2
	dV/dcos(phi) = -2*k*(1+phi*phi/6.)

   If |sin(phi)|.lt.1e-03 and (phi-Peq).lt.0.1
   and cnseq(phi)=pi
	V = kimp (phi - impeq)^2
	dV/dcos(phi) = 2*k*(1+phi*phi/6.)

   Now, if |sin(phi)|.lt.1e-03 but far from minimum,
   set sin(phi) to 1e-03 arbitrarily

*/

	REAL_T e;
	REAL_T co, phi, den, co1;
	REAL_T uu, vv, uv;
	REAL_T ax, bx, cx;
	REAL_T ay, by, cy;
	REAL_T az, bz, cz;
	REAL_T ux, uy, uz;
	REAL_T vx, vy, vz;
	REAL_T dx1, dy1, dz1;
	REAL_T a0x, b0x, c0x;
	REAL_T a0y, b0y, c0y;
	REAL_T a0z, b0z, c0z;
	REAL_T a1x, b1x;
	REAL_T a1y, b1y;
	REAL_T a1z, b1z;
	REAL_T a2x, b2x;
	REAL_T a2y, b2y;
	REAL_T a2z, b2z;
	REAL_T dd1x, dd2x, dd3x, dd4x;
	REAL_T dd1y, dd2y, dd3y, dd4y;
	REAL_T dd1z, dd2z, dd3z, dd4z;
	REAL_T df, delta, pi, yy;
	REAL_T aa, bb, cc, ab, bc, ac;
	REAL_T e_imp;
	int i;
	int at1, at2, at3, at4, atyp;

	pi = 3.1415927;

	e_imp = 0.0;
	for (i = 0; i < nphi; i++) {

		at1 = a1[i];
		at2 = a2[i];
		at3 = abs(a3[i]);
		at4 = abs(a4[i]);
		atyp = atype[i] - 1;


/*	calculate distances  */

		ax = x[at2 + 0] - x[at1 + 0];
		ay = x[at2 + 1] - x[at1 + 1];
		az = x[at2 + 2] - x[at1 + 2];
		bx = x[at3 + 0] - x[at2 + 0];
		by = x[at3 + 1] - x[at2 + 1];
		bz = x[at3 + 2] - x[at2 + 2];
		cx = x[at4 + 0] - x[at3 + 0];
		cy = x[at4 + 1] - x[at3 + 1];
		cz = x[at4 + 2] - x[at3 + 2];

#		define DOT(a,b,c,d,e,f) a*d + b*e + c*f
		ab = DOT(ax, ay, az, bx, by, bz);
		bc = DOT(bx, by, bz, cx, cy, cz);
		ac = DOT(ax, ay, az, cx, cy, cz);
		aa = DOT(ax, ay, az, ax, ay, az);
		bb = DOT(bx, by, bz, bx, by, bz);
		cc = DOT(cx, cy, cz, cx, cy, cz);


		uu = (aa * bb) - (ab * ab);
		vv = (bb * cc) - (bc * bc);
		uv = (ab * bc) - (ac * bb);

		den = 1.0 / sqrt(fabs(uu * vv));
		co = uv * den;
		co = co > 1.0 ? 1.0 : co;
		co = co < -1.0 ? -1.0 : co;

		phi = acos(co);

/*
	now calculate sin(phi) because cos(phi) is symmetric, so
	we can decide between +-phi.
*/

		ux = ay * bz - az * by;
		uy = az * bx - ax * bz;
		uz = ax * by - ay * bx;

		vx = by * cz - bz * cy;
		vy = bz * cx - bx * cz;
		vz = bx * cy - by * cx;

		dx1 = uy * vz - uz * vy;
		dy1 = uz * vx - ux * vz;
		dz1 = ux * vy - uy * vx;

		dx1 = DOT(dx1, dy1, dz1, bx, by, bz);
		if (dx1 < 0.0)
			phi = -phi;

		delta = phi - Peq[atyp];
		delta = delta > pi ? pi : delta;
		delta = delta < -pi ? -pi : delta;

		df = Pk[atyp] * delta;
		e = df * delta;
		e_imp += e;
		yy = sin(phi);

/*
	Decide what expansion to use
	Check first for the "normal" expression, since it will be
	the most used

	the 0.001 value could be lowered for increased precision.
	This insures ~1e-05% error for sin(phi)=0.001
*/

		if (fabs(yy) > 0.001) {
			df = -2.0 * df / yy;
		} else {
			if (fabs(delta) < 0.10) {
				if (Peq[atyp] == 0.0) {
					df = -2.0 * Pk[atyp] * (1 + phi * phi / 6.0);
				} else {
					if (fabs(Peq[atyp]) == pi)
						df = 2.0 * Pk[atyp] * (1 + delta * delta / 6.0);
				}
			} else {
				if ((phi > 0.0 && phi < (pi / 2.0)) ||
					(phi < 0.0 && phi > -pi / 2.0))
					df = df * 1000.;
				else
					df = -df * 1000.;
			}
		}

		co1 = 0.5 * co * den;

		a0x = -bc * bx + bb * cx;
		a0y = -bc * by + bb * cy;
		a0z = -bc * bz + bb * cz;

		b0x = ab * cx + bc * ax - 2. * ac * bx;
		b0y = ab * cy + bc * ay - 2. * ac * by;
		b0z = ab * cz + bc * az - 2. * ac * bz;

		c0x = ab * bx - bb * ax;
		c0y = ab * by - bb * ay;
		c0z = ab * bz - bb * az;

		a1x = 2. * uu * (-cc * bx + bc * cx);
		a1y = 2. * uu * (-cc * by + bc * cy);
		a1z = 2. * uu * (-cc * bz + bc * cz);

		b1x = 2. * uu * (bb * cx - bc * bx);
		b1y = 2. * uu * (bb * cy - bc * by);
		b1z = 2. * uu * (bb * cz - bc * bz);

		a2x = -2. * vv * (bb * ax - ab * bx);
		a2y = -2. * vv * (bb * ay - ab * by);
		a2z = -2. * vv * (bb * az - ab * bz);

		b2x = 2. * vv * (aa * bx - ab * ax);
		b2y = 2. * vv * (aa * by - ab * ay);
		b2z = 2. * vv * (aa * bz - ab * az);

		dd1x = (a0x - a2x * co1) * den;
		dd1y = (a0y - a2y * co1) * den;
		dd1z = (a0z - a2z * co1) * den;

		dd2x = (-a0x - b0x - (a1x - a2x - b2x) * co1) * den;
		dd2y = (-a0y - b0y - (a1y - a2y - b2y) * co1) * den;
		dd2z = (-a0z - b0z - (a1z - a2z - b2z) * co1) * den;

		dd3x = (b0x - c0x - (-a1x - b1x + b2x) * co1) * den;
		dd3y = (b0y - c0y - (-a1y - b1y + b2y) * co1) * den;
		dd3z = (b0z - c0z - (-a1z - b1z + b2z) * co1) * den;

		dd4x = (c0x - b1x * co1) * den;
		dd4y = (c0y - b1y * co1) * den;
		dd4z = (c0z - b1z * co1) * den;

		f[at1 + 0] += df * dd1x;
		f[at1 + 1] += df * dd1y;
		f[at1 + 2] += df * dd1z;

		f[at2 + 0] += df * dd2x;
		f[at2 + 1] += df * dd2y;
		f[at2 + 2] += df * dd2z;

		f[at3 + 0] += df * dd3x;
		f[at3 + 1] += df * dd3y;
		f[at3 + 2] += df * dd3z;

		f[at4 + 0] += df * dd4x;
		f[at4 + 1] += df * dd4y;
		f[at4 + 2] += df * dd4z;

	}

	return (e_imp);
}

int nblist( REAL_T *x, int *npairs, int *pairlist )
{
	REAL_T dx, dy, dz, rrw, cut2;
	int tot_pair;
	int nx, ipair, ji, ires, jres, ip1, jp1, ip2, jp2, i, j, k,
		jrp, jrp2, kpr;
	int *ires_pairlist;
	int *iexw;

	ires_pairlist = ivector(0, prm->Nres);
	iexw = ivector(-1, prm->Natom);
/*
    loop over all pairs of residues:
*/
	tot_pair = 0;
	kpr = 0;
	cut2 = cut * cut;
	ji = 0;
	for (i = 0; i < prm->Natom; i++) {
		iexw[i] = -1;
	}
	for (ires = 0; ires < prm->Nres; ires++) {
		ip1 = prm->Ipres[ires];
		ip2 = prm->Ipres[ires + 1] - 1;
		jrp = 0;
		ires_pairlist[0] = ires;
		for (jres = ires + 1; jres < prm->Nres; jres++) {
			jp1 = prm->Ipres[jres];
			jp2 = prm->Ipres[jres + 1] - 1;
			for (i = ip1 - 1; i < ip2; i++) {
				for (j = jp1 - 1; j < jp2; j++) {
					dx = x[3 * i + 0] - x[3 * j + 0];
					dy = x[3 * i + 1] - x[3 * j + 1];
					dz = x[3 * i + 2] - x[3 * j + 2];
					rrw = dx * dx + dy * dy + dz * dz;
					if (rrw < cut2)
						goto includit;
				}
			}
			continue;
		  includit:ires_pairlist[++jrp] = jres;
		}
/*
     ---- End of generating the resdue pair list for residue ires 
     ---- Now take care of exclusions and sort the lists:
*/
		for (i = ip1 - 1; i < ip2; i++) {
			ipair = 0;
			nx = prm->Iblo[i];
			for (k = 0; k < nx; k++) {
				iexw[prm->ExclAt[ji + k] - 1] = i;
			}
			ji += nx;
			for (jrp2 = 0; jrp2 <= jrp; jrp2++) {
				jres = ires_pairlist[jrp2];
				if (ires == jres)
					jp1 = i + 2;
				else
					jp1 = prm->Ipres[jres];
				jp2 = prm->Ipres[jres + 1] - 1;
				for (j = jp1 - 1; j < jp2; j++) {
					if (iexw[j] != i && (!frozen[i] || !frozen[j])) {
						ipair++;
						pairlist[kpr++] = j;
					}
				}
			}
			tot_pair += ipair;
			npairs[i] = ipair;
			if (tot_pair > maxnb) {
				fprintf(stderr, "maxnb (%u) is too small\n", maxnb);
				exit(1);
			}
		}
	}

#ifdef PRINT_NB_LIST
	printf("npairs:\n");
	for (u = 0; u < prm->Natom; u++) {
		printf("%4d", npairs[u]);
		if ((u + 1) % 20 == 0)
			printf("\n");
	}
	printf("\npairlist:\n");
	for (u = 0; u < tot_pair; u++) {
		printf("%4d", pairlist[u]);
		if ((u + 1) % 20 == 0)
			printf("\n");
	}
	printf("\n");
#endif

	free_ivector(ires_pairlist, 0, prm->Nres);
	free_ivector(iexw, -1, prm->Natom);

	printf("                              ");
	printf("                              ");
	printf("        %u\n", tot_pair);
	return (tot_pair);
}

int nbond( int *npairs, int *pairlist,
	REAL_T *x, REAL_T *f, REAL_T *enb, REAL_T *eel,
	REAL_T enbfac, REAL_T eelfac )
{
	int i, j, jn, ic, npr, lpair, iaci;
	REAL_T dumx, dumy, dumz, cgi, xw1, xw2, xw3, r2inv, df2, r6, r10, f1,
		f2;
	REAL_T df, fw1, fw2, fw3, enbfaci, eelfaci;
	REAL_T dis, kij, d0, diff, rinv, rs, rssq, eps1, epsi, cgijr, pow;
	int nhbpair, ibig, isml;
/*	REAL_T	hbener, nb14;  */

/*	nhbpair = 0; hbener = 0.0; nb14 = 0.0;  */

#define SIG 0.3
#define DIW 78.0
#define C1 38.5

	lpair = 0;
	*enb = 0.;
	*eel = 0.;
	enbfaci = 1. / enbfac;
	eelfaci = 1. / eelfac;
	for (i = 0; i < prm->Natom - 1; i++) {

		npr = npairs[i];
		if (npr > 0) {
			iaci = prm->Ntypes * (prm->Iac[i] - 1);
			dumx = 0.;
			dumy = 0.;
			dumz = 0.;
			cgi = eelfaci * prm->Charges[i];

			if (dield == -3) {
				/* special code RL dielectric, 94 force field */
				for (jn = 0; jn < npr; jn++) {
					j = pairlist[jn + lpair];
					xw1 = x[3 * i + 0] - x[3 * j + 0];
					xw2 = x[3 * i + 1] - x[3 * j + 1];
					xw3 = x[3 * i + 2] - x[3 * j + 2];
					r2inv = 1. / (xw1 * xw1 + xw2 * xw2 + xw3 * xw3);

					/* Ramstein & Lavery dielectric,
					   PNAS 85, 7231 (1988).         */
					rinv = sqrt(r2inv);
					rs = SIG / rinv;
					rssq = rs * rs;
					pow = exp(-rs);
					eps1 = rssq + rs + rs + 2.0;
					epsi = 1.0 / (DIW - C1 * pow * eps1);
					cgijr = cgi * prm->Charges[j] * rinv * epsi;
					*eel += cgijr;
					df2 = -cgijr * (1.0 + C1 * pow * rs * rssq * epsi);

					ic = prm->Cno[iaci + prm->Iac[j] - 1] - 1;
					r6 = r2inv * r2inv * r2inv;
					f2 = prm->Cn2[ic] * r6;
					f1 = prm->Cn1[ic] * r6 * r6;

/*
**					if( f1 > 500000. )
**						fprintf( stderr, "close contact: %d %d %8.3f\n", 
**						i+1,j+1,1./rinv );
*/

					*enb += (f1 - f2) * enbfaci;
					df = (df2 + 6. * (f2 - f1 - f1) * enbfaci) * r2inv;
					fw1 = xw1 * df;
					fw2 = xw2 * df;
					fw3 = xw3 * df;
					dumx = dumx + fw1;
					dumy = dumy + fw2;
					dumz = dumz + fw3;
					f[3 * j + 0] -= fw1;
					f[3 * j + 1] -= fw2;
					f[3 * j + 2] -= fw3;
				}
			} else if (dield == -4) {
				/* distance-dependent dielectric code, 94 ff */
				for (jn = 0; jn < npr; jn++) {
					j = pairlist[jn + lpair];
					xw1 = x[3 * i + 0] - x[3 * j + 0];
					xw2 = x[3 * i + 1] - x[3 * j + 1];
					xw3 = x[3 * i + 2] - x[3 * j + 2];
					r2inv = 1. / (xw1 * xw1 + xw2 * xw2 + xw3 * xw3);

					/* epsilon = r  */
					rs = cgi * prm->Charges[j] * r2inv;
					df2 = -rs - rs;
					*eel += rs;
					ic = prm->Cno[iaci + prm->Iac[j] - 1] - 1;
					r6 = r2inv * r2inv * r2inv;
					f2 = prm->Cn2[ic] * r6;
					f1 = prm->Cn1[ic] * r6 * r6;
					*enb += (f1 - f2) * enbfaci;
					df = (df2 + 6. * (f2 - f1 - f1) * enbfaci) * r2inv;
					fw1 = xw1 * df;
					fw2 = xw2 * df;
					fw3 = xw3 * df;
					dumx = dumx + fw1;
					dumy = dumy + fw2;
					dumz = dumz + fw3;
					f[3 * j + 0] -= fw1;
					f[3 * j + 1] -= fw2;
					f[3 * j + 2] -= fw3;
				}
			} else if (dield == -5) {
				/* non-bonded term from yammp  */
				for (jn = 0; jn < npr; jn++) {
					j = pairlist[jn + lpair];
					xw1 = x[3 * i + 0] - x[3 * j + 0];
					xw2 = x[3 * i + 1] - x[3 * j + 1];
					xw3 = x[3 * i + 2] - x[3 * j + 2];
					dis = sqrt(xw1 * xw1 + xw2 * xw2 + xw3 * xw3);

					ic = prm->Cno[iaci + prm->Iac[j] - 1] - 1;
					d0 = prm->Cn2[ic];
					if (dis < d0) {
						kij = prm->Cn1[ic];
						diff = dis - d0;
						*enb += kij * diff * diff;
						df = 2. * kij * diff / dis;
						fw1 = xw1 * df;
						fw2 = xw2 * df;
						fw3 = xw3 * df;
						dumx = dumx + fw1;
						dumy = dumy + fw2;
						dumz = dumz + fw3;
						f[3 * j + 0] -= fw1;
						f[3 * j + 1] -= fw2;
						f[3 * j + 2] -= fw3;
					}
				}
			} else {
				/* standard code with all options   */
				for (jn = 0; jn < npr; jn++) {
					j = pairlist[jn + lpair];
					xw1 = x[3 * i + 0] - x[3 * j + 0];
					xw2 = x[3 * i + 1] - x[3 * j + 1];
					xw3 = x[3 * i + 2] - x[3 * j + 2];
					r2inv = 1. / (xw1 * xw1 + xw2 * xw2 + xw3 * xw3);

					/*  Code for various dielectric models.
					   The df2 variable should hold r(dV/dr).    */

					if (dield == 0) {
						/* epsilon = r  */
						rs = cgi * prm->Charges[j] * r2inv;
						df2 = -rs - rs;
						*eel += rs;
					} else if (dield == 1) {
						/* epsilon = 1  */
						rinv = sqrt(r2inv);
						df2 = -cgi * prm->Charges[j] * rinv;
						*eel -= df2;

					} else if (dield == -2) {
						/* Ramstein & Lavery dielectric,
						   PNAS 85, 7231 (1988).         */
						rinv = sqrt(r2inv);
						rs = SIG / rinv;
						rssq = rs * rs;
						pow = exp(-rs);
						eps1 = rssq + rs + rs + 2.0;
						epsi = 1.0 / (DIW - C1 * pow * eps1);
						cgijr = cgi * prm->Charges[j] * rinv * epsi;
						*eel += cgijr;
						df2 = -cgijr * (1.0 + C1 * pow * rs * rssq * epsi);
					}
					ic = prm->Cno[iaci + prm->Iac[j] - 1];
					if (ic > 0 || enbfac != 1.0) {
						if (ic > 0) {
							ic--;
						} else {
							ibig = prm->Iac[i] > prm->Iac[j] ?
								prm->Iac[i] : prm->Iac[j];
							isml = prm->Iac[i] > prm->Iac[j] ?
								prm->Iac[j] : prm->Iac[i];
							ic = ibig * (ibig - 1) / 2 + isml - 1;
						}
						r6 = r2inv * r2inv * r2inv;
						f2 = prm->Cn2[ic] * r6;
						f1 = prm->Cn1[ic] * r6 * r6;

/*
**						if( f1 > 500000. )
**							fprintf( stderr, "close contact: %d %d  %8.3f\n", 
**							i+1,j+1,1./rinv );
*/

						*enb += (f1 - f2) * enbfaci;
						df = (df2 + 6. * (f2 - f1 - f1) * enbfaci) * r2inv;
/*						if( enbfac != 1.0 ) nb14 += (f1-f2)*enbfaci;    */
					} else {
						ic = -ic - 1;
						r10 = r2inv * r2inv * r2inv * r2inv * r2inv;
						f2 = prm->HB10[ic] * r10;
						f1 = prm->HB12[ic] * r10 * r2inv;
						*enb += (f1 - f2) * enbfaci;
						df = (df2 +
							  (10. * f2 - 12. * f1) * enbfaci) * r2inv;
						nhbpair++;
/*						 hbener += (f1-f2)*enbfaci;                     */
					}
					fw1 = xw1 * df;
					fw2 = xw2 * df;
					fw3 = xw3 * df;
					dumx = dumx + fw1;
					dumy = dumy + fw2;
					dumz = dumz + fw3;
					f[3 * j + 0] -= fw1;
					f[3 * j + 1] -= fw2;
					f[3 * j + 2] -= fw3;
				}
			}
			f[3 * i + 0] += dumx;
			f[3 * i + 1] += dumy;
			f[3 * i + 2] += dumz;
			lpair += npr;
		}
	}
/*
	printf( "found %d hydrogen-bond pairs with energy %.3f\n", 
		nhbpair, hbener );
	printf( "1-4 nonbon = %.3f\n", nb14 );
*/
	return (0);
}

REAL_T egb( int *natom,
	REAL_T x[], REAL_T f[], REAL_T fs[], REAL_T rborn[], REAL_T q[],
	REAL_T *kappa, REAL_T *diel_ext, REAL_T *enb, REAL_T *eelt )
/*
 *	int *natom		number of atoms
 *	REAL_T x[]		input: coordinates
 *	REAL_T f[]		forces: updated on output
 *	REAL_T fs[]		input: overlap parameters
 *	REAL_T rborn[]		input: atomic radii
 *	REAL_T q[]		input: atomic charges
 *	REAL_T *kappa		inverse of Debye-Huckel length
 *	REAL_T *diel_ext	solvent dielectric constant
 *	REAL_T *enb		Lennard-Jones energy
 *	REAL_T *eelt		gas-phase electrostatic energy
*/

#define BOFFSET 0.09
#define KSCALE 0.73

#define PIx4 12.5663706143591724639918
#define PIx2  6.2831853071795862319959
#define PIx1  3.1415926535897931159979

{

	int i, j, k, size_da;
	int ic, iaci, skip, iexcl, jexcl, jexcl_last, nexcl;
	REAL_T e, epol, dielfac, qi, qiqj, fgbi, fgbk, rb2, expmkf;
	REAL_T xi, yi, zi, xij, yij, zij;
	REAL_T dedx, dedy, dedz, de;
	REAL_T dij1i, dij3i, temp1;
	REAL_T qi2h, qid2h, datmp;
	REAL_T daix, daiy, daiz, theta, ri1i, dij2i, ri2i;

	REAL_T dij, sumi;
	REAL_T cut2, eel, f6, f12, rinv, r2inv, r6inv;
	REAL_T r2, ri, rj, sj, sj2, thi, thi2;
	REAL_T uij, efac, temp4, temp5, temp6, temp7;
	REAL_T *reff, *da, *sumdeijda, *psi;

	/* LCPO stuff follows */
	int count, count2, icount;
	REAL_T si, sumAij, sumAjk, sumAijAjk, sumdAijddijdxi;
	REAL_T sumdAijddijdyi, sumdAijddijdzi, sumdAijddijdxiAjk;
	REAL_T sumdAijddijdyiAjk, sumdAijddijdziAjk, rij, tmpaij, Aij,
		dAijddij;
	REAL_T dAijddijdxj, dAijddijdyj, dAijddijdzj;
	REAL_T sumdAjkddjkdxj, sumdAjkddjkdyj, sumdAjkddjkdzj, p3p4Aij;
	REAL_T xk, yk, zk, rjk2, djk1i, rjk, vdw2dif, tmpajk, Ajk, sumAjk2,
		dAjkddjk;
	REAL_T dAjkddjkdxj, dAjkddjkdyj, dAjkddjkdzj, lastxj, lastyj, lastzj;
	REAL_T dAidxj, dAidyj, dAidzj, Ai, Aidxi, Aidyi, Aidzi, dAidxi, dAidyi,
		dAidzi;
	REAL_T totsasa;
	REAL_T xj, yj, zj;
	REAL_T dumbo,tmpsd;
	REAL_T rgbmax2, rgbmax1i, rgbmax2i, tmpcs, rgbmaxpsmax2;

  /*FGB taylor coefficients follow */
  /* from A to H :                 */
  /* 1/3 , 2/5 , 3/7 , 4/9 , 5/11  */
  /* 4/3 , 12/5 , 24/7 , 40/9 , 60/11 */

#define TA 0.33333333333333333333
#define TB 0.4
#define TC 0.42857142857142857143
#define TD 0.44444444444444444444
#define TDD 0.45454545454545454545

#define TE 1.33333333333333333333
#define TF 2.4
#define TG 3.42857142857142857143
#define TH 4.44444444444444444444
#define THH 5.45454545454545454545

	/* Smooth "cut-off" in calculating GB effective radii.
	 Implementd by Andreas Svrcek-Seiler and Alexey Onufriev.
	 The integration over solute is performed up to rgbmax and includes
	 parts of spheres; that is an atom is not just "in" or "out", as
	 with standard non-bonded cut.  As a result, calclated effective
	 radii are less than rgbmax. This saves time, and there is no
	 discontinuity in dReff/drij.

	 Only the case rgbmax > 5*max(sij) = 5*fsmax ~ 9A is handled; this is
	 enforced in mdread().  Smaller values would not make much physical
	 sense anyway.  */

	rgbmax2 = rgbmax*rgbmax;
	rgbmax1i = 1.0/rgbmax;
	rgbmax2i = rgbmax1i*rgbmax1i;
	rgbmaxpsmax2 = (rgbmax+prm->Fsmax)*(rgbmax+prm->Fsmax);

	reff = vector(0, *natom);
	if( gb==2 | gb==5 ) psi = vector(0, *natom);

	size_da = 3 * (*natom);
	da = vector(0, size_da);

	sumdeijda = vector(0, *natom);
	for (i = 0; i < *natom; i++)
		sumdeijda[i] = 0.0;

	count = 0;
	totsasa = 0.0;
	cut2 = cut * cut;


/*    get the "effective" Born radii via the approximate pairwise method
      Use Eqs 9-11 of Hawkins, Cramer, Truhlar, J. Phys. Chem. 100:19824
      (1996).  Below, lij and uij and the inverses of the L and U 
      parameters defined in this reference.                             */

	if (gb_debug)
		printf("Effective Born radii:\n");

	for (i = 0; i < *natom; i++) {

		xi = x[3 * i];
		yi = x[3 * i + 1];
		zi = x[3 * i + 2];
		ri = rborn[i] - BOFFSET;
		ri1i = 1. / ri;
		sumi = 0.;

		for (j = 0; j < *natom; j++) {
			if (i == j)
				continue;
			xij = xi - x[3 * j];
			yij = yi - x[3 * j + 1];
			zij = zi - x[3 * j + 2];
			r2 = xij * xij + yij * yij + zij * zij;
			if( r2 > rgbmaxpsmax2 ) continue;
			dij1i = 1.0 / sqrt(r2);
			dij = r2 * dij1i;
			sj = fs[j] * (rborn[j] - BOFFSET);
			sj2 = sj * sj;

/*         ---following are from the Appendix of Schaefer and Froemmel,
            JMB 216:1045-1066, 1990;  Taylor series expansion for d>>s
			is by Andreas Svrcek-Seiler; smooth rgbmax idea is from
			Andreas Svrcek-Seiler and Alexey Onufriev                 */

			if( dij > rgbmax + sj ) continue;

			if ((dij > rgbmax - sj)) {
               uij = 1./(dij -sj);
               sumi -= 0.125 * dij1i * (1.0 + 2.0 * dij *uij +
                     rgbmax2i * (r2 - 4.0 * rgbmax * dij - sj2) +
                     2.0 * log((dij-sj)*rgbmax1i));

			} else if (dij >4.0*sj) {
				dij2i = dij1i*dij1i;
				tmpsd = sj2*dij2i;
				dumbo = TA+tmpsd*(TB+tmpsd*(TC+tmpsd*(TD+tmpsd*TDD)));
				sumi -= sj*tmpsd*dij2i*dumbo;

			} else if (dij > ri + sj) {
				sumi -= 0.5 * (sj / (r2 - sj2) +
						   0.5 * dij1i * log((dij - sj) / (dij + sj)));

			} else if (dij > fabs(ri - sj)) {
				theta = 0.5 * ri1i * dij1i * (r2 + ri * ri - sj2);
				uij = 1. / (dij + sj);
				sumi -= 0.25 * (ri1i * (2. - theta) - uij +
							dij1i * log(ri * uij));

			} else if (ri < sj) {
				sumi -= 0.5 * (sj / (r2 - sj2) + 2. * ri1i +
							   0.5 * dij1i * log((sj - dij) / (sj + dij)));

			}

			if (gbsa == 1) {
				if ((P0[i] + P0[j]) > dij) {
					if (P0[i] > 2.5 && P0[j] > 2.5) {

						ineighbor[count] = j + 1;
						/* this +1 is VERY important */
						count += 1;
					}
				}
			}

		}

		if( gb==1 ){

			/* "standard" (HCT) effective radii:  */
			reff[i] = 1.0/(ri1i + sumi);

		} else {

			/* "gbao" formulas:  */

			psi[i] = -ri*sumi;
			reff[i] = 1.0 / (ri1i - tanh( (gbalpha - gbbeta*psi[i] + 
						gbgamma*psi[i]*psi[i]) *psi[i])/rborn[i] );
		}

		if (gbsa == 1) {
			ineighbor[count] = 0;
			count = count + 1;
		}

		/*      reff[ i ] = rborn[i] > reff[i] ?  rborn[i] : reff[i];   */

		if (gb_debug)
			printf("%d\t%15.7f\t%15.7f\n", i + 1, rborn[i], reff[i]);
	}


	/* main LCPO stuff follows */

	if (gbsa == 1) {
		totsasa = 0.0;
		count = 0;
		for (i = 0; i < *natom; i++) {
			if (ineighbor[count] == 0) {
				count = count + 1;
			} else {

				si = PIx4 * P0[i] * P0[i];
				sumAij = 0.0;
				sumAjk = 0.0;
				sumAjk2 = 0.0;
				sumAijAjk = 0.0;
				sumdAijddijdxi = 0.0;
				sumdAijddijdyi = 0.0;
				sumdAijddijdzi = 0.0;
				sumdAijddijdxiAjk = 0.0;
				sumdAijddijdyiAjk = 0.0;
				sumdAijddijdziAjk = 0.0;

				icount = count;

			  L70:j = ineighbor[count] - 1;
				/* mind the -1 , fortran again */
				xi = x[3 * i];
				yi = x[3 * i + 1];
				zi = x[3 * i + 2];

				xj = x[3 * j];
				yj = x[3 * j + 1];
				zj = x[3 * j + 2];



				r2 = (xi - xj) * (xi - xj) + (yi - yj) * (yi - yj) + (zi -
																	  zj) *
					(zi - zj);
				dij1i = 1. / sqrt(r2);
				rij = r2 * dij1i;
				tmpaij =
					P0[i] - rij * 0.5 - (P0[i] * P0[i] -
										 P0[j] * P0[j]) * 0.5 * dij1i;

				Aij = PIx2 * P0[i] * tmpaij;

				dAijddij =
					PIx1 * P0[i] * (dij1i * dij1i *
									(P0[i] * P0[i] - P0[j] * P0[j]) - 1.0);

				dAijddijdxj = dAijddij * (xj - xi) * dij1i;
				dAijddijdyj = dAijddij * (yj - yi) * dij1i;
				dAijddijdzj = dAijddij * (zj - zi) * dij1i;

				sumAij = sumAij + Aij;

				count2 = icount;
				sumAjk2 = 0.0;
				sumdAjkddjkdxj = 0.0;
				sumdAjkddjkdyj = 0.0;
				sumdAjkddjkdzj = 0.0;

				p3p4Aij = -surften * (P3[i] + P4[i] * Aij);

			  L80:k = ineighbor[count2] - 1;
				/*same as above, -1 ! */
				if (j == k)
					goto L85;

				xk = x[3 * k];
				yk = x[3 * k + 1];
				zk = x[3 * k + 2];

				rjk2 =
					(xj - xk) * (xj - xk) + (yj - yk) * (yj - yk) + (zj -
																	 zk) *
					(zj - zk);

				djk1i = 1.0 / sqrt(rjk2);

				rjk = rjk2 * djk1i;

				if (P0[j] + P0[k] > rjk) {
					vdw2dif = P0[j] * P0[j] - P0[k] * P0[k];
					tmpajk = 2.0 * P0[j] - rjk - vdw2dif * djk1i;

					Ajk = PIx1 * P0[j] * tmpajk;

					sumAjk = sumAjk + Ajk;
					sumAjk2 = sumAjk2 + Ajk;

					dAjkddjk =
						PIx1 * P0[j] * djk1i * (djk1i * djk1i * vdw2dif -
												1.0);

					dAjkddjkdxj = dAjkddjk * (xj - xk);
					dAjkddjkdyj = dAjkddjk * (yj - yk);
					dAjkddjkdzj = dAjkddjk * (zj - zk);

					f[3 * k] = f[3 * k] + dAjkddjkdxj * p3p4Aij;
					f[3 * k + 1] = f[3 * k + 1] + dAjkddjkdyj * p3p4Aij;
					f[3 * k + 2] = f[3 * k + 2] + dAjkddjkdzj * p3p4Aij;

					sumdAjkddjkdxj = sumdAjkddjkdxj + dAjkddjkdxj;
					sumdAjkddjkdyj = sumdAjkddjkdyj + dAjkddjkdyj;
					sumdAjkddjkdzj = sumdAjkddjkdzj + dAjkddjkdzj;

				}

			  L85:count2 = count2 + 1;
				if (ineighbor[count2] != 0) {
					goto L80;
				} else {
					count2 = icount;
				}


				sumAijAjk = sumAijAjk + Aij * sumAjk2;

				sumdAijddijdxi = sumdAijddijdxi - dAijddijdxj;
				sumdAijddijdyi = sumdAijddijdyi - dAijddijdyj;
				sumdAijddijdzi = sumdAijddijdzi - dAijddijdzj;
				sumdAijddijdxiAjk =
					sumdAijddijdxiAjk - dAijddijdxj * sumAjk2;
				sumdAijddijdyiAjk =
					sumdAijddijdyiAjk - dAijddijdyj * sumAjk2;
				sumdAijddijdziAjk =
					sumdAijddijdziAjk - dAijddijdzj * sumAjk2;

				lastxj = dAijddijdxj * sumAjk2 + Aij * sumdAjkddjkdxj;
				lastyj = dAijddijdyj * sumAjk2 + Aij * sumdAjkddjkdyj;
				lastzj = dAijddijdzj * sumAjk2 + Aij * sumdAjkddjkdzj;

				dAidxj =
					surften * (P2[i] * dAijddijdxj +
							   P3[i] * sumdAjkddjkdxj + P4[i] * lastxj);
				dAidyj =
					surften * (P2[i] * dAijddijdyj +
							   P3[i] * sumdAjkddjkdyj + P4[i] * lastyj);
				dAidzj =
					surften * (P2[i] * dAijddijdzj +
							   P3[i] * sumdAjkddjkdzj + P4[i] * lastzj);

				f[3 * j] = f[3 * j] + dAidxj;
				f[3 * j + 1] = f[3 * j + 1] + dAidyj;
				f[3 * j + 2] = f[3 * j + 2] + dAidzj;

				count = count + 1;
				if (ineighbor[count] != 0) {
					goto L70;
				} else {
					count = count + 1;
				}

				Ai = P1[i] * si + P2[i] * sumAij + P3[i] * sumAjk +
					P4[i] * sumAijAjk;

				dAidxi =
					surften * (P2[i] * sumdAijddijdxi +
							   P4[i] * sumdAijddijdxiAjk);
				dAidyi =
					surften * (P2[i] * sumdAijddijdyi +
							   P4[i] * sumdAijddijdyiAjk);
				dAidzi =
					surften * (P2[i] * sumdAijddijdzi +
							   P4[i] * sumdAijddijdziAjk);

				f[3 * i] = f[3 * i] + dAidxi;
				f[3 * i + 1] = f[3 * i + 1] + dAidyi;
				f[3 * i + 2] = f[3 * i + 2] + dAidzi;

				totsasa = totsasa + Ai;
				/*  printf("totsasa up to %d %f\n",i,totsasa); */
			}
		}
		/*      printf("SASA %f , ESURF %f \n",totsasa,totsasa*surften); */
		esurf = totsasa * surften;
	}

/*    Compute the GB, Coulomb and Lennard-Jones energies and derivatives:  */

	epol = *eelt = *enb = 0.0;
	iexcl = 0;

	for (i = 0; i < *natom; i++) {
		xi = x[3 * i];
		yi = x[3 * i + 1];
		zi = x[3 * i + 2];
		qi = q[i];
		ri = reff[i];
		iaci = prm->Ntypes * (prm->Iac[i] - 1);
		jexcl = iexcl;
		jexcl_last = iexcl + prm->Iblo[i] - 1;
		nexcl = prm->ExclAt[jexcl] - 1;
		if (jexcl > jexcl_last)
			nexcl = -1;

		for (j = i + 1; j < *natom; j++) {

			skip = 0;
			if (j == nexcl) {
				skip = 1;
				jexcl++;
				nexcl = prm->ExclAt[jexcl] - 1;
				if (jexcl > jexcl_last)
					nexcl = -1;
			}

			xij = xi - x[3 * j];
			yij = yi - x[3 * j + 1];
			zij = zi - x[3 * j + 2];
			r2 = xij * xij + yij * yij + zij * zij;
			if( r2 > cut2 ) continue;

			qiqj = qi * q[j];
			rj = reff[j];
			rb2 = ri * rj;
			efac = exp(-r2 / (4.0 * rb2));
			fgbi = 1.0 / sqrt(r2 + rb2 * efac);
			fgbk = -(*kappa) * KSCALE / fgbi;

			expmkf = exp(fgbk) / (*diel_ext);
			dielfac = 1.0 - expmkf;

			e = -qiqj * dielfac * fgbi;
			epol += e;

			temp4 = fgbi * fgbi * fgbi;
			temp6 = -qiqj * temp4 * (dielfac + fgbk * expmkf);
			de = -temp6 * (1.0 - 0.25 * efac);

			temp5 = 0.5 * efac * temp6 * (rb2 + 0.25 * r2);
			sumdeijda[i] += ri * temp5;
			sumdeijda[j] += rj * temp5;

			if (gb_debug > 1 && e != 0.0) {
				printf("%4d %4d %10.5f %10.5f %10.5f %10.5f %10.5f\n",
					   i + 1, j + 1, e, q[i] / 18.2223, q[j] / 18.2223,
					   1.0 / fgbi, sqrt(r2));
			}

			if (!skip) {

				rinv = 1. / sqrt(r2);
				r2inv = rinv * rinv;

				/*  gas-phase Coulomb energy:  */

				eel = qiqj * rinv;
				*eelt += eel;
				de -= eel * r2inv;

				/* Lennard-Jones energy:   */

				ic = prm->Cno[iaci + prm->Iac[j] - 1] - 1;
				r6inv = r2inv * r2inv * r2inv;
				f6 = prm->Cn2[ic] * r6inv;
				f12 = prm->Cn1[ic] * r6inv * r6inv;
				*enb += f12 - f6;
				de -= (12. * f12 - 6. * f6) * r2inv;
			}

			dedx = de * xij;
			dedy = de * yij;
			dedz = de * zij;
			f[3 * i] += dedx;
			f[3 * i + 1] += dedy;
			f[3 * i + 2] += dedz;
			f[3 * j] -= dedx;
			f[3 * j + 1] -= dedy;
			f[3 * j + 2] -= dedz;
		}

		iexcl += prm->Iblo[i];

	}

	/*  Add in derivatives with respect to effective radius, plus
	   the "diagonal" terms                                         */

	for (i = 0; i < *natom; i++) {
		expmkf = exp(-KSCALE * (*kappa) * reff[i]) / (*diel_ext);
		dielfac = 1.0 - expmkf;
		qi2h = 0.5 * q[i] * q[i];
		qid2h = qi2h * dielfac;
		epol -= qid2h / reff[i];
		temp7 =
			-sumdeijda[i] + qid2h -
			KSCALE * (*kappa) * qi2h * expmkf * reff[i];

		xi = x[3 * i];
		yi = x[3 * i + 1];
		zi = x[3 * i + 2];
		ri = rborn[i] - BOFFSET;
		ri1i = 1. / ri;
		daix = daiy = daiz = 0.0;

		for (j = 0; j < *natom; j++) {
			if (i == j)
				continue;
			da[3 * j] = da[3 * j + 1] = da[3 * j + 2] = 0.0;
			xij = xi - x[3 * j];
			yij = yi - x[3 * j + 1];
			zij = zi - x[3 * j + 2];
			r2 = xij * xij + yij * yij + zij * zij;
			if( r2 > rgbmaxpsmax2 ) continue;
			dij1i = 1.0 / sqrt(r2);
			dij2i = dij1i * dij1i;
			dij = r2 * dij1i;
			sj = fs[j] * (rborn[j] - BOFFSET);
			sj2 = sj * sj;

			if( dij > rgbmax + sj ) continue;

			if( dij > rgbmax - sj ){
				temp1 = 1./(dij-sj);
               datmp = 0.125 * dij3i * ((r2 + sj2) *
                       (temp1*temp1 - rgbmax2i) - 2.0 * log(rgbmax*temp1));

			} else if(dij >4.0*sj) {
				tmpsd = sj2*dij2i;
				dumbo = TE + tmpsd*(TF+tmpsd*(TG+tmpsd*(TH+tmpsd*THH)));
				datmp = tmpsd*sj*dij2i*dij2i*dumbo;

			} else if (dij > ri + sj) {

				temp1 = 1. / (r2 - sj2);
				datmp = temp1 * sj * (-0.5 * dij2i + temp1)
					+ 0.25 * dij1i * dij2i * log((dij - sj) / (dij + sj));

			} else if (dij > fabs(ri - sj)) {

				temp1 = 1. / (dij + sj);
				dij3i = dij1i * dij1i * dij1i;
				datmp =
					-0.25 * (-0.5 * (r2 - ri * ri + sj2) * dij3i * ri1i *
							 ri1i + dij1i * temp1 * (temp1 - dij1i)
							 - dij3i * log(ri * temp1));

			} else if (ri < sj) {

				temp1 = 1. / (r2 - sj2);
				datmp =
					-0.5 * (sj * dij2i * temp1 - 2. * sj * temp1 * temp1 -
							0.5 * dij2i * dij1i * log((sj - dij) /
													  (sj + dij)));

			} else {
				datmp = 0.;
			}

/*                 da(3*j-2) = d (alpha-i)/ d (x-j), etc.
                  [where alpha is the contribution to the inverse
                   of the effective radius]                           */

			da[3 * j] = -xij * datmp;
			da[3 * j + 1] = -yij * datmp;
			da[3 * j + 2] = -zij * datmp;
			daix -= da[3 * j];
			daiy -= da[3 * j + 1];
			daiz -= da[3 * j + 2];

		}
		da[3 * i] = daix;
		da[3 * i + 1] = daiy;
		da[3 * i + 2] = daiz;

		if( gb>1 ){

			ri = rborn[i] - BOFFSET;
			thi = tanh( (gbalpha - gbbeta*psi[i] + gbgamma*psi[i]*psi[i])
					*psi[i] );
			temp7 *= (gbalpha -2.0*gbbeta*psi[i] + 3.0*gbgamma*psi[i]*psi[i])
					*(1.0 - thi*thi)*ri/rborn[i];
		}

		for (k = 0; k < *natom * 3; k++) f[k] -= temp7 * da[k];

	}

	if (gb_debug > 1) {
		printf("gb    epol= %10.3f\n", epol);
		for (k = 0; k < *natom; k++) {
			printf("%15.7f\n", sumdeijda[k]);
		}
		for (k = 0; k < *natom * 3; k += 3) {
			printf("%15.7f%15.7f%15.7f\n", f[k], f[k + 1], f[k + 2]);
		}
	}

	free_vector(reff, 0, *natom);
	if( gb == 2 | gb==5 ) free_vector(psi, 0, *natom);
	free_vector(sumdeijda, 0, *natom);
	free_vector(da, 0, size_da);

	return (epol);
}

