REAL_T	ebond4( int, int *, int *, int *, REAL_T *, REAL_T *, REAL_T *, REAL_T * );
REAL_T	eangl4( int, int *, int *, int *, int *, REAL_T *, REAL_T *, REAL_T *, REAL_T * );
REAL_T	ephi4( int, int *, int *, int *, int *, int *, REAL_T *, REAL_T *, REAL_T *, REAL_T *, REAL_T * );
int	nblist4( REAL_T *, int *, int * );
int	nbond4( int *, int *, REAL_T *, REAL_T *, REAL_T *, REAL_T *, REAL_T, REAL_T );
REAL_T	mme4( REAL_T *,  REAL_T *, int * );

REAL_T	ebond4( int nbond, int *a1, int *a2, int *atype,
	REAL_T *Rk, REAL_T *Req, REAL_T *x, REAL_T *f )
/*
int nbond, *a1, *a2, *atype;
REAL_T *Rk, *Req, *x, *f;
*/
{
	int i, at1, at2, atyp;
	REAL_T e_bond, r, rx, ry, rz, rw, r2, s, db, df, e;

	e_bond = 0.0;
	for (i = 0; i < nbond; i++) {
		at1 = 4 * a1[i] / 3;
		at2 = 4 * a2[i] / 3;
		atyp = atype[i] - 1;
		rx = x[at1] - x[at2];
		ry = x[at1 + 1] - x[at2 + 1];
		rz = x[at1 + 2] - x[at2 + 2];
		rw = x[at1 + 3] - x[at2 + 3];
		r2 = rx * rx + ry * ry + rz * rz + rw * rw;
		s = sqrt(r2);
		r = 2.0 / s;
		db = s - Req[atyp];
		df = Rk[atyp] * db;
		e = df * db;
		e_bond += e;
		df *= r;
		f[at1 + 0] += rx * df;
		f[at1 + 1] += ry * df;
		f[at1 + 2] += rz * df;
		f[at1 + 3] += rw * df;
		f[at2 + 0] -= rx * df;
		f[at2 + 1] -= ry * df;
		f[at2 + 2] -= rz * df;
		f[at2 + 3] -= rw * df;
	}
	return (e_bond);
}

REAL_T	eangl4( int nang, int *a1, int *a2, int *a3, int *atype,
	REAL_T *Tk, REAL_T *Teq, REAL_T *x, REAL_T *f )
/*
int nang, *a1, *a2, *a3, *atype;
REAL_T *Tk, *Teq, *x, *f;
*/
{
	int i, atyp, at1, at2, at3;
	REAL_T dxi, dyi, dzi, dwi, dxj, dyj, dzj, dwj, ri2, rj2, ri, rj, rir,
		rjr;
	REAL_T dxir, dyir, dzir, dwir, dxjr, dyjr, dzjr, dwjr, cst, at, da, df,
		e, e_theta;
	REAL_T xtmp, dxtmp, ytmp, wtmp, dytmp, ztmp, dztmp, dwtmp;

	e_theta = 0.0;
	for (i = 0; i < nang; i++) {
		at1 = 4 * a1[i] / 3;
		at2 = 4 * a2[i] / 3;
		at3 = 4 * a3[i] / 3;
		atyp = atype[i] - 1;

		dxi = x[at1] - x[at2];
		dyi = x[at1 + 1] - x[at2 + 1];
		dzi = x[at1 + 2] - x[at2 + 2];
		dwi = x[at1 + 3] - x[at2 + 3];
		dxj = x[at3] - x[at2];
		dyj = x[at3 + 1] - x[at2 + 1];
		dzj = x[at3 + 2] - x[at2 + 2];
		dwj = x[at3 + 3] - x[at2 + 3];

		ri2 = dxi * dxi + dyi * dyi + dzi * dzi + dwi * dwi;
		rj2 = dxj * dxj + dyj * dyj + dzj * dzj + dwj * dwj;
		ri = sqrt(ri2);
		rj = sqrt(rj2);
		rir = 1. / ri;
		rjr = 1. / rj;

		dxir = dxi * rir;
		dyir = dyi * rir;
		dzir = dzi * rir;
		dwir = dwi * rir;
		dxjr = dxj * rjr;
		dyjr = dyj * rjr;
		dzjr = dzj * rjr;
		dwjr = dwj * rjr;

		cst = dxir * dxjr + dyir * dyjr + dzir * dzjr + dwir * dwjr;
		if (cst > 1.0)
			cst = 1.0;
		if (cst < -1.0)
			cst = -1.0;

		at = acos(cst);
		da = at - Teq[atyp];
		df = da * Tk[atyp];
		e = df * da;
		e_theta = e_theta + e;
		df = df + df;
		at = sin(at);
		if (at > 0 && at < 1.e-3)
			at = 1.e-3;
		else if (at < 0 && at > -1.e-3)
			at = -1.e-3;
		df = -df / at;

		xtmp = df * rir * (dxjr - cst * dxir);
		dxtmp = df * rjr * (dxir - cst * dxjr);
		ytmp = df * rir * (dyjr - cst * dyir);
		dytmp = df * rjr * (dyir - cst * dyjr);
		ztmp = df * rir * (dzjr - cst * dzir);
		dztmp = df * rjr * (dzir - cst * dzjr);
		wtmp = df * rir * (dwjr - cst * dwir);
		dwtmp = df * rjr * (dwir - cst * dwjr);

		f[at1 + 0] += xtmp;
		f[at3 + 0] += dxtmp;
		f[at2 + 0] -= xtmp + dxtmp;
		f[at1 + 1] += ytmp;
		f[at3 + 1] += dytmp;
		f[at2 + 1] -= ytmp + dytmp;
		f[at1 + 2] += ztmp;
		f[at3 + 2] += dztmp;
		f[at2 + 2] -= ztmp + dztmp;
		f[at1 + 3] += wtmp;
		f[at3 + 3] += dwtmp;
		f[at2 + 3] -= wtmp + dwtmp;

	}
	return (e_theta);
}

REAL_T	ephi4( int nphi, int *a1, int *a2, int *a3, int *a4, int *atype,
	REAL_T *Pk, REAL_T *Pn, REAL_T *Phase, REAL_T *x, REAL_T *f )
/*
int nphi, *a1, *a2, *a3, *a4, *atype;
REAL_T *Pk, *Pn, *Phase, *x, *f;
*/
{
	REAL_T e, co, den, co1, uu, vv, uv, ax, bx, cx, ay, by, cy, az, bz, cz,
		aw, bw, cw;
	REAL_T a0x, b0x, c0x, a0y, b0y, c0y, a0z, b0z, c0z, a0w, b0w, c0w, a1x,
		b1x;
	REAL_T a1y, b1y, a1z, b1z, a1w, b1w, a2x, b2x, a2y, b2y, a2z, b2z, a2w,
		b2w;
	REAL_T dd1x, dd2x, dd3x, dd4x, dd1y, dd2y, dd3y, dd4y, dd1z, dd2z,
		dd3z, dd4z;
	REAL_T dd1w, dd2w, dd3w, dd4w;
	REAL_T df, aa, bb, cc, ab, bc, ac, ktot;
	REAL_T ktors1, ktors2, ktors3, ktors4, phase, e_tors;
	int i, at1, at2, at3, at4, atyp;
	REAL_T ux, uy, uz, vx, vy, vz, delta, phi, dx1, dy1, dz1, yy, pi;
	pi = 3.1415927;

	e_tors = 0.0;
	for (i = 0; i < nphi; i++) {

		at1 = 4 * a1[i] / 3;
		at2 = 4 * a2[i] / 3;
		at3 = 4 * abs(a3[i]) / 3;
		at4 = 4 * abs(a4[i]) / 3;
		atyp = atype[i] - 1;

		ax = x[at2 + 0] - x[at1 + 0];
		ay = x[at2 + 1] - x[at1 + 1];
		az = x[at2 + 2] - x[at1 + 2];
		aw = x[at2 + 3] - x[at1 + 3];
		bx = x[at3 + 0] - x[at2 + 0];
		by = x[at3 + 1] - x[at2 + 1];
		bz = x[at3 + 2] - x[at2 + 2];
		bw = x[at3 + 3] - x[at2 + 3];
		cx = x[at4 + 0] - x[at3 + 0];
		cy = x[at4 + 1] - x[at3 + 1];
		cz = x[at4 + 2] - x[at3 + 2];
		cw = x[at4 + 3] - x[at3 + 3];

#		define DOT4(a,b,c,d,e,f,g,h) a*e + b*f + c*g + d*h
		ab = DOT4(ax, ay, az, aw, bx, by, bz, bw);
		bc = DOT4(bx, by, bz, bw, cx, cy, cz, cw);
		ac = DOT4(ax, ay, az, aw, cx, cy, cz, cw);
		aa = DOT4(ax, ay, az, aw, ax, ay, az, aw);
		bb = DOT4(bx, by, bz, bw, bx, by, bz, bw);
		cc = DOT4(cx, cy, cz, cw, cx, cy, cz, cw);

		uu = (aa * bb) - (ab * ab);
		vv = (bb * cc) - (bc * bc);
		uv = (ab * bc) - (ac * bb);
		den = 1.0 / sqrt(uu * vv);
		co = uv * den;
		co1 = 0.5 * co * den;

		a0x = -bc * bx + bb * cx;
		a0y = -bc * by + bb * cy;
		a0z = -bc * bz + bb * cz;
		a0w = -bc * bw + bb * cw;

		b0x = ab * cx + bc * ax - 2. * ac * bx;
		b0y = ab * cy + bc * ay - 2. * ac * by;
		b0z = ab * cz + bc * az - 2. * ac * bz;
		b0w = ab * cw + bc * aw - 2. * ac * bw;

		c0x = ab * bx - bb * ax;
		c0y = ab * by - bb * ay;
		c0z = ab * bz - bb * az;
		c0w = ab * bw - bb * aw;

		a1x = 2. * uu * (-cc * bx + bc * cx);
		a1y = 2. * uu * (-cc * by + bc * cy);
		a1z = 2. * uu * (-cc * bz + bc * cz);
		a1w = 2. * uu * (-cc * bw + bc * cw);

		b1x = 2. * uu * (bb * cx - bc * bx);
		b1y = 2. * uu * (bb * cy - bc * by);
		b1z = 2. * uu * (bb * cz - bc * bz);
		b1w = 2. * uu * (bb * cw - bc * bw);

		a2x = -2. * vv * (bb * ax - ab * bx);
		a2y = -2. * vv * (bb * ay - ab * by);
		a2z = -2. * vv * (bb * az - ab * bz);
		a2w = -2. * vv * (bb * aw - ab * bw);

		b2x = 2. * vv * (aa * bx - ab * ax);
		b2y = 2. * vv * (aa * by - ab * ay);
		b2z = 2. * vv * (aa * bz - ab * az);
		b2w = 2. * vv * (aa * bw - ab * aw);

		dd1x = (a0x - a2x * co1) * den;
		dd1y = (a0y - a2y * co1) * den;
		dd1z = (a0z - a2z * co1) * den;
		dd1w = (a0w - a2w * co1) * den;

		dd2x = (-a0x - b0x - (a1x - a2x - b2x) * co1) * den;
		dd2y = (-a0y - b0y - (a1y - a2y - b2y) * co1) * den;
		dd2z = (-a0z - b0z - (a1z - a2z - b2z) * co1) * den;
		dd2w = (-a0w - b0w - (a1w - a2w - b2w) * co1) * den;

		dd3x = (b0x - c0x - (-a1x - b1x + b2x) * co1) * den;
		dd3y = (b0y - c0y - (-a1y - b1y + b2y) * co1) * den;
		dd3z = (b0z - c0z - (-a1z - b1z + b2z) * co1) * den;
		dd3w = (b0w - c0w - (-a1w - b1w + b2w) * co1) * den;

		dd4x = (c0x - b1x * co1) * den;
		dd4y = (c0y - b1y * co1) * den;
		dd4z = (c0z - b1z * co1) * den;
		dd4w = (c0w - b1w * co1) * den;

		if (prm->Nhparm && a3[i] < 0) {
/*   here we will use a quadratic form for the improper torsion  */
/*     we are using the NHPARM variable in prmtop to trigger this   */
/*
	WARNING: phi itself is here calculated from the first three coords--
     --- may fail!
*/

			co = co > 1.0 ? 1.0 : co;
			co = co < -1.0 ? -1.0 : co;

			phi = acos(co);

/*
	now calculate sin(phi) because cos(phi) is symmetric, so
	we can decide between +-phi.
*/

			ux = ay * bz - az * by;
			uy = az * bx - ax * bz;
			uz = ax * by - ay * bx;

			vx = by * cz - bz * cy;
			vy = bz * cx - bx * cz;
			vz = bx * cy - by * cx;

			dx1 = uy * vz - uz * vy;
			dy1 = uz * vx - ux * vz;
			dz1 = ux * vy - uy * vx;

			dx1 = DOT(dx1, dy1, dz1, bx, by, bz);
			if (dx1 < 0.0)
				phi = -phi;

			delta = phi - Phase[atyp];
			delta = delta > pi ? pi : delta;
			delta = delta < -pi ? -pi : delta;

			df = Pk[atyp] * delta;
			e = df * delta;
			e_tors += e;
			yy = sin(phi);

/*
	Decide what expansion to use
	Check first for the "normal" expression, since it will be
	the most used

	the 0.001 value could be lowered for increased precision.
	This insures ~1e-05% error for sin(phi)=0.001
*/

			if (fabs(yy) > 0.001) {
				df = -2.0 * df / yy;
			} else {
				if (fabs(delta) < 0.10) {
					if (Phase[atyp] == 0.0) {
						df = -2.0 * Pk[atyp] * (1 + phi * phi / 6.0);
					} else {
						if (fabs(Phase[atyp]) == pi) {
							df = 2.0 * Pk[atyp] * (1 +
												   delta * delta / 6.0);
						}
					}
				} else {
					if ((phi > 0.0 && phi < (pi / 2.0)) ||
						(phi < 0.0 && phi > -pi / 2.0))
						df = df * 1000.;
					else
						df = -df * 1000.;
				}
			}
		} else {
		  multi_term:
			ktors1 = 0.;
			ktors2 = 0.;
			ktors3 = 0.;
			ktors4 = 0.;
			switch ((int) fabs(Pn[atyp])) {
			case 1:
				ktors1 = Pk[atyp];
				break;
			case 2:
				ktors2 = Pk[atyp];
				break;
			case 3:
				ktors3 = Pk[atyp];
				break;
			case 4:
				ktors4 = Pk[atyp];
				break;
			default:
				fprintf(stderr, "bad value for Pk: %d %d %d %d %8.3f\n",
						at1, at2, at3, at4, Pn[atyp]);
				exit(1);
			}
			if (fabs(Phase[atyp] - 3.142) < 0.01)
				phase = -1.0;
			else
				phase = 1.0;

			e = 4. * ktors3 * co * co + 2. * ktors2 * co + ktors1 -
				3. * ktors3 + 8. * ktors4 * co * (co * co - 1.);
			e = e * co - ktors2 + ktors4;
			ktot = ktors1 + ktors2 + ktors3 + ktors4;
			e = ktot + phase * e;
			e_tors += e;

			df = phase * (12. * ktors3 * co * co + 4. * ktors2 * co +
						  ktors1 - 3. * ktors3 +
						  32. * ktors4 * co * co * co - 16. * ktors4 * co);
		}

		f[at1 + 0] += df * dd1x;
		f[at1 + 1] += df * dd1y;
		f[at1 + 2] += df * dd1z;
		f[at1 + 3] += df * dd1w;

		f[at2 + 0] += df * dd2x;
		f[at2 + 1] += df * dd2y;
		f[at2 + 2] += df * dd2z;
		f[at2 + 3] += df * dd2w;

		f[at3 + 0] += df * dd3x;
		f[at3 + 1] += df * dd3y;
		f[at3 + 2] += df * dd3z;
		f[at3 + 3] += df * dd3w;

		f[at4 + 0] += df * dd4x;
		f[at4 + 1] += df * dd4y;
		f[at4 + 2] += df * dd4z;
		f[at4 + 3] += df * dd4w;

#ifdef PRINT_EPHI
		printf("%4d%4d%4d%4d%4d%8.3f\n", i + 1, at1, at2, at3, at4, e);
		printf("%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f%10.5f\n",
			   -df * dd1x, -df * dd1y, -df * dd1z, -df * dd2x, -df * dd2y,
			   -df * dd2z, -df * dd3x, -df * dd3y);
		printf("%10.5f%10.5f%10.5f%10.5f\n", -df * dd3z, -df * dd4x,
			   -df * dd4y, -df * dd4z);
#endif
		if (Pn[atyp] < 0.0) {
			atyp++;
			goto multi_term;
		}
	}
	return (e_tors);
}

int	nblist4( REAL_T *x, int *npairs, int *pairlist )
/*
REAL_T *x;
int *npairs, *pairlist;
*/
{
	REAL_T dx, dy, dz, dw, rrw, cut2;
	int tot_pair, u;
	int nx, ipair, ji, ires, jres, ip1, jp1, ip2, jp2, i, j, k,
		jrp, jrp2, kpr;
	int *ires_pairlist;
	int *iexw;

	ires_pairlist = ivector(0, prm->Nres);
	iexw = ivector(-1, prm->Natom);
/*
    loop over all pairs of residues:
*/
	tot_pair = 0;
	kpr = 0;
	cut2 = cut * cut;
	ji = 0;
	for (i = 0; i < prm->Natom; i++) {
		iexw[i] = -1;
	}
	for (ires = 0; ires < prm->Nres; ires++) {
		ip1 = prm->Ipres[ires];
		ip2 = prm->Ipres[ires + 1] - 1;
		jrp = 0;
		ires_pairlist[0] = ires;
		for (jres = ires + 1; jres < prm->Nres; jres++) {
			jp1 = prm->Ipres[jres];
			jp2 = prm->Ipres[jres + 1] - 1;
			for (i = ip1 - 1; i < ip2; i++) {
				for (j = jp1 - 1; j < jp2; j++) {
					dx = x[4 * i + 0] - x[4 * j + 0];
					dy = x[4 * i + 1] - x[4 * j + 1];
					dz = x[4 * i + 2] - x[4 * j + 2];
					dw = x[4 * i + 3] - x[4 * j + 3];
					rrw = dx * dx + dy * dy + dz * dz + dw * dw;
					if (rrw < cut2)
						goto includit;
				}
			}
			continue;
		  includit:ires_pairlist[++jrp] = jres;
		}
/*
     ---- End of generating the resdue pair list for residue ires 
     ---- Now take care of exclusions and sort the lists:
*/
		for (i = ip1 - 1; i < ip2; i++) {
			ipair = 0;
			nx = prm->Iblo[i];
			for (k = 0; k < nx; k++) {
				iexw[prm->ExclAt[ji + k] - 1] = i;
			}
			ji += nx;
			for (jrp2 = 0; jrp2 <= jrp; jrp2++) {
				jres = ires_pairlist[jrp2];
				if (ires == jres)
					jp1 = i + 2;
				else
					jp1 = prm->Ipres[jres];
				jp2 = prm->Ipres[jres + 1] - 1;
				for (j = jp1 - 1; j < jp2; j++) {
					if (iexw[j] != i && (!frozen[i] || !frozen[j])) {
						ipair++;
						pairlist[kpr++] = j;
					}
				}
			}
			tot_pair += ipair;
			npairs[i] = ipair;
			if (tot_pair > maxnb) {
				fprintf(stderr, "maxnb (%d) is too small\n", maxnb);
				exit(1);
			}
		}
	}

#ifdef PRINT_NB_LIST
	printf("npairs:\n");
	for (u = 0; u < prm->Natom; u++) {
		printf("%4d", npairs[u]);
		if ((u + 1) % 20 == 0)
			printf("\n");
	}
	printf("\npairlist:\n");
	for (u = 0; u < tot_pair; u++) {
		printf("%4d", pairlist[u]);
		if ((u + 1) % 20 == 0)
			printf("\n");
	}
	printf("\n");
#endif

	free_ivector(ires_pairlist, 0, prm->Nres);
	free_ivector(iexw, -1, prm->Natom);

	printf("                              ");
	printf("                              ");
	printf("        %u\n", tot_pair);
	return (tot_pair);
}

int	nbond4( int *npairs, int *pairlist,
	REAL_T *x, REAL_T *f, REAL_T *enb, REAL_T *eel, REAL_T enbfac, REAL_T eelfac )
/*
int *npairs, *pairlist;
REAL_T *x, *f;
REAL_T *enb, *eel;
REAL_T enbfac, eelfac;
*/
{
	int i, j, jn, ic, npr, lpair, iaci;
	REAL_T dumx, dumy, dumz, dumw, cgi, xw1, xw2, xw3, xw4, r2inv, df2, r6,
		f1, f2;
	REAL_T df, fw1, fw2, fw3, fw4, enbfaci, eelfaci, r10;
	int ibig, isml;

	lpair = 0;
	*enb = 0.;
	*eel = 0.;
	enbfaci = 1. / enbfac;
	eelfaci = 1. / eelfac;
	for (i = 0; i < prm->Natom - 1; i++) {

		npr = npairs[i];
		if (npr > 0) {
			iaci = prm->Ntypes * (prm->Iac[i] - 1);
			dumx = 0.;
			dumy = 0.;
			dumz = 0.;
			dumw = 0.;
			cgi = -2. * prm->Charges[i];
			for (jn = 0; jn < npr; jn++) {
				j = pairlist[jn + lpair];
				xw1 = x[4 * i + 0] - x[4 * j + 0];
				xw2 = x[4 * i + 1] - x[4 * j + 1];
				xw3 = x[4 * i + 2] - x[4 * j + 2];
				xw4 = x[4 * i + 3] - x[4 * j + 3];
				r2inv =
					1. / (xw1 * xw1 + xw2 * xw2 + xw3 * xw3 + xw4 * xw4);
				df2 = eelfaci * cgi * prm->Charges[j] * r2inv;
				*eel -= df2;
				ic = prm->Cno[iaci + prm->Iac[j] - 1];
				if (ic > 0 || enbfac != 1.0) {
					if (ic > 0) {
						ic--;
					} else {
						ibig = prm->Iac[i] > prm->Iac[j] ?
							prm->Iac[i] : prm->Iac[j];
						isml = prm->Iac[i] > prm->Iac[j] ?
							prm->Iac[j] : prm->Iac[i];
						ic = ibig * (ibig - 1) / 2 + isml - 1;
					}
					r6 = r2inv * r2inv * r2inv;
					f2 = prm->Cn2[ic] * r6;
					f1 = prm->Cn1[ic] * r6 * r6;
					*enb += (f1 - f2) * enbfaci;
					df = (df2 + 6. * (f2 - f1 - f1) * enbfaci) * r2inv;
				} else {
					ic = -ic - 1;
					r10 = r2inv * r2inv * r2inv * r2inv * r2inv;
					f2 = prm->HB10[ic] * r10;
					f1 = prm->HB12[ic] * r10 * r2inv;
					*enb += (f1 - f2) * enbfaci;
					df = (df2 + (10. * f2 - 12. * f1) * enbfaci) * r2inv;
				}
				fw1 = xw1 * df;
				fw2 = xw2 * df;
				fw3 = xw3 * df;
				fw4 = xw4 * df;
				dumx += fw1;
				dumy += fw2;
				dumz += fw3;
				dumw += fw4;
				f[4 * j + 0] -= fw1;
				f[4 * j + 1] -= fw2;
				f[4 * j + 2] -= fw3;
				f[4 * j + 3] -= fw4;
			}
			f[4 * i + 0] += dumx;
			f[4 * i + 1] += dumy;
			f[4 * i + 2] += dumz;
			f[4 * i + 3] += dumw;
			lpair += npr;
		}
	}
	*eel *= 0.5;
	return (0);
}

REAL_T	mme4( REAL_T *x,  REAL_T *f, int *iter )
/*
REAL_T *x, *f;
int *iter;
*/
{
	REAL_T ebh, eba, eth, eta, eph, epa, enb, eel, enb14, eel14, frms;
	REAL_T ene[20];
	int i, k;
	REAL_T err;

	if (*iter <= 1) {
		printf("    iter   bond    angle    dihed.     vdW    elect.");
		printf("   e4d      Total     grms\n");
	}
	if (nb_pairs < 0 || (*iter % nsnb == 0 && *iter != 0)) {
		int firstlist = nb_pairs < 0;
		nb_pairs = nblist4(x, npairs, pairlist);
	}

	for (i = 0; i < 4 * prm->Natom; i++)
		f[i] = 0.0;

	nbond4(npairs, pairlist, x, f, &enb, &eel, 1.0, 1.0);
	ene[1] = enb;
	ene[2] = eel;

	ebh = ebond4(prm->Nbonh, prm->BondHAt1, prm->BondHAt2,
				 prm->BondHNum, prm->Rk, prm->Req, x, f);
	eba = ebond4(prm->Mbona, prm->BondAt1, prm->BondAt2,
				 prm->BondNum, prm->Rk, prm->Req, x, f);
	ene[3] = ebh + eba;

	eth = eangl4(prm->Ntheth, prm->AngleHAt1, prm->AngleHAt2,
				 prm->AngleHAt3, prm->AngleHNum, prm->Tk, prm->Teq, x, f);
	eta = eangl4(prm->Ntheta, prm->AngleAt1, prm->AngleAt2,
				 prm->AngleAt3, prm->AngleNum, prm->Tk, prm->Teq, x, f);
	ene[4] = eth + eta;

	eph = ephi4(prm->Nphih, prm->DihHAt1, prm->DihHAt2,
				prm->DihHAt3, prm->DihHAt4, prm->DihHNum,
				prm->Pk, prm->Pn, prm->Phase, x, f);
	epa = ephi4(prm->Mphia, prm->DihAt1, prm->DihAt2,
				prm->DihAt3, prm->DihAt4, prm->DihNum,
				prm->Pk, prm->Pn, prm->Phase, x, f);
	ene[5] = eph + epa;

	ene[6] = 0.0;				/*  hbond term not in Amber-94 force field */

	nbond4(prm->N14pairs, prm->N14pairlist, x, f, &enb14, &eel14,
		   scnb, scee);
	ene[7] = enb14;
	ene[8] = eel14;

	ene[9] = 0.0;
	if (k4d != 0.0) {
		for (i = 0; i < prm->Natom; i++) {
			err = x[4 * i + 3];
			f[4 * i + 3] += k4d * err;
			ene[9] += 0.5 * k4d * err * err;
		}
	}

	ene[0] = 0.0;
	for (k = 1; k <= 9; k++)
		ene[0] += ene[k];

	for (k = 0; k < prm->Natom; k++) {	/* zero out frozen forces */
		if (frozen[k]) {
			f[4 * k + 0] = 0.0;
			f[4 * k + 1] = 0.0;
			f[4 * k + 2] = 0.0;
			f[4 * k + 3] = 0.0;
		}
	}

	frms = 0.0;
	for (i = 0; i < 4 * prm->Natom; i++)
		frms += f[i] * f[i];
	frms = sqrt(frms / (4 * prm->Natom));

/*	for( i=0; i<4*prm->Natom; i+=4 )
 *		printf( "%10.5f%10.5f%10.5f%10.5f\n", f[i],f[i+1],f[i+2],f[i+3] );
*/

	if (*iter == 1 || *iter % ntpr == 0) {
		printf("ff:%4d%9.2f%9.2f%9.2f%9.2f%9.2f%9.2f%9.2f%9.2f\n",
			   *iter, ene[3], ene[4], ene[5], ene[1] + ene[7],
			   ene[2] + ene[8], ene[9], ene[0], frms);
	}

	return (ene[0]);
}
